import os
import signal
import concurrent.futures
import time

def read_last_number(file_path):
    """Read the last number from the file, default to 6000111 if file does not exist or is empty."""
    if os.path.exists(file_path):
        with open(file_path, 'r') as file:
            content = file.read().strip()
            if content:
                return int(content)
    return 6000111  # Default starting number

def write_last_number(file_path, number):
    """Write the new number to the file."""
    with open(file_path, 'w') as file:
        file.write(str(number))

def download_file(base_url, number):
    url = f"{base_url}{number}"

    # Use wget to download the file directly to the current directory
    result = os.system(
        f"wget --method=GET -nc --no-check-certificate "
        f"--header='Host: coregateway.app.dlight.com' "
        f"--header='User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:128.0) Gecko/20100101 Firefox/128.0' "
        f"--header='Accept: application/json, text/plain, */*' "
        f"--header='Accept-Language: en-US,en;q=0.5' "
        f"--header='Accept-Encoding: gzip, deflate, br' "
        f"--header='Authorization: Bearer eyJhbGciOiJIUzUxMiJ9.eyJpYXQiOjE3MjMyOTM5MTQsImlzcyI6Imh0dHBzOi8vd3d3LmRsaWdodC5jb20vIiwic3ViIjoiNTA1ODEiLCJDTEFJTV9UT0tFTiI6IlJPTEVfRUMgaW52ZW50b3J5IFVwZGF0ZWQtMTAwMS0xMDAxLFJPTEVfRXhwZXJpZW5jZSBDZW50ZXItMTAwMS0xMDAxIiwiZXhwIjoxNzIzMzgwMzE0fQ.S1K2NLplf8IsbitiRg563oIwd4z4j-Meo40J2zdIA4wxOM629HB_HRdrXps7CPHTG1KdU6gb0HKNzc29lnB0fA' "
        f"--header='Origin: https://atlas.app.dlight.com' "
        f"--header='Dnt: 1' "
        f"--header='Sec-Gpc: 1' "
        f"--header='Referer: https://atlas.app.dlight.com/' "
        f"--content-disposition -P ./files_dlight/ {url}"
    )
    
    return result == 0  # Return True if download was successful

def handle_signal(signum, frame):
    """Handle termination signals."""
    print(f"Received signal {signum}, cleaning up and exiting.")
    # Perform cleanup if necessary
    exit(0)

def process_download(last_number_file, base_url, max_workers=200):
    """Download files concurrently, starting from the given number."""
    # Register signal handlers for termination signals
    signal.signal(signal.SIGTERM, handle_signal)
    signal.signal(signal.SIGINT, handle_signal)

    # Read the starting number from the file
    start_number = read_last_number(last_number_file)

    # Initialize a list to hold futures
    futures = []

    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
        number = start_number

        while True:
            # Submit a download task to the executor
            future = executor.submit(download_file, base_url, number)
            futures.append(future)

            # Check if any futures are done and update the last number
            for future in futures:
                if future.done():
                    if future.result():
                        print(f"Successfully downloaded number: {number}")
                        write_last_number(last_number_file, number + 1)
                    else:
                        print(f"Failed to download number: {number}")

                    # Remove completed future from list
                    futures.remove(future)
                    number += 1

            # Optional: Small sleep to prevent CPU overuse
            time.sleep(0.1)

# Define the path to the last number file and the base URL
last_number_file = 'last_number.txt'
base_url = 'https://coregateway.app.dlight.com/coreservicecrm/api/v1/customerdocumentattachment/download/'

# Process downloads with concurrency
process_download(last_number_file, base_url, max_workers=200)
